@@ -4,7 +4,7 @@
 ### dependencies: pks, stringr
 ###
 
-read_kstructure <- function (filename, format="auto") {
+read_kstructure <- function (filename, format="auto", as.letters = TRUE) {
 
   f <- readLines(con=filename)
   if (length(f) == 0) {
@@ -24,6 +24,11 @@ read_kstructure <- function (filename, format="auto") {
     if (nos <= 0)
       stop(sprintf("Invalid number of states in %s.", filename))
     offset <- 3
+    p <- str_locate(f[4], "#")
+    while (!is.na(p[1][1])) {
+      offset <- offset + 1
+      p <- str_locate(f[offset+1], "#")
+    }
   }
   else if (format == "KST") {
     noi <- as.numeric(f[1])
@@ -54,6 +59,11 @@ read_kstructure <- function (filename, format="auto") {
       if (nos <= 0)
         stop(sprintf("Invalid number of states in %s.", filename))
       offset <- 3
+      p <- str_locate(f[4], "#")
+      while (!is.na(p[1][1])) {
+        offset <- offset + 1
+        p <- str_locate(f[offset+1], "#")
+      }
     }
     else if (str_length(f[1]) == str_length(f[length(f)])) { # most probably matrix
       nos <- length(f)
@@ -72,12 +82,19 @@ read_kstructure <- function (filename, format="auto") {
   }   # end of automatic format detection
 
   mat <- mat.or.vec(nos, noi)
-  storage.mode(mat) <- "integer"
   for (i in 1:nos) {
-    mat[i,]<- 1*as.logical(as.numeric(unlist(strsplit(trimws(f[i+offset],which="both"),""))))
+    mat[i,]<- 1L*as.logical(as.integer(unlist(strsplit(trimws(f[i+offset],which="both"),""))))
   }
+  storage.mode(mat) <- "integer"
+  if (as.letters) {
+    names <- make.unique(letters[(0L:(ncol(mat)-1)) %% 26 + 1])
+  } else {
+    names <- as.integer(1L:ncol(mat))
+  }
+  colnames(mat) <- names
+  
   s <- as.pattern(mat, as.set=TRUE)
-  class(s) <- c("kstructure", class(s))
+  class(s) <- unique(c("kstructure", "kfamset", class(s)))
   
   list(matrix=mat, sets=s)
   
